package com.dayou.controller;

import com.dayou.annotation.OperLog;
import com.dayou.dto.ItemDTO;
import com.dayou.vo.DepartmentStatVO;
import com.dayou.vo.ItemStatVO;
import com.dayou.vo.ItemVO;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.controller.BaseController;
import com.dayou.service.IItemService;
import com.dayou.entity.Item;
import com.dayou.common.RestResponse;
import org.springframework.web.bind.annotation.*;
import com.dayou.common.PullDownModel;
import com.dayou.utils.ConvertUtil;
import com.dayou.utils.HttpKit;
import com.dayou.exception.ErrorCode;
import java.util.Date;
import java.util.List;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.http.MediaType;
import org.springframework.web.multipart.MultipartFile;
/**
 * 项目信息表
 *
 * @author wucl
 * @since 2022-12-08
 */
@RestController
@RequestMapping("item")
@Slf4j
public class ItemController extends BaseController {
    @Autowired
    private IItemService itemService;

    /**
    * 项目信息表列表
    */
    @GetMapping("")
    public RestResponse<Page<ItemVO>> page(Item item, Page page, Boolean me){
        Page<ItemVO> pages=itemService.selectPage(page,item,me);
        return RestResponse.data(pages);
    }

    /**
     * 项目信息表详情
     */
    @GetMapping("/{id}")
    public RestResponse<ItemVO> detail(@PathVariable Long id){
        ItemVO itemVO =itemService.detail(id);
        return RestResponse.data(itemVO);
     }

    /**
     * 项目信息表新增
     */
    @OperLog(title = "新增项目")
    @PostMapping("")
    public RestResponse<Boolean> save(@RequestBody ItemDTO itemDTO) {
        Boolean ret = itemService.add(itemDTO);
        return RestResponse.data(ret);
    }

    /**
     * 项目信息表更新
     */
    @OperLog(title = "更新项目")
    @PutMapping("")
    public RestResponse<Boolean> update(@RequestBody ItemDTO itemDTO) {
        Boolean ret = itemService.update(itemDTO);
        return RestResponse.data(ret);
    }

    /**
     * 项目信息表删除
     */
    @OperLog(title = "删除项目")
    @DeleteMapping("/{id}")
    public RestResponse<Boolean> delete(@PathVariable Long id) {
        Boolean ret = itemService.delete(id);
        return RestResponse.data(ret);
    }


    /**
     * 项目信息表下拉列表
     */
    @GetMapping("/simpleAll")
    public RestResponse<List<PullDownModel>> simpleAll(){
        LambdaQueryWrapper<Item> lambdaQueryWrapper = new LambdaQueryWrapper<Item>()
        .select(Item::getId,Item::getName)
        .eq(Item::getDeleted,false);
        List<Item> list= itemService.list(lambdaQueryWrapper);
        List<PullDownModel> ret=ConvertUtil.copyList(list, PullDownModel.class);
        return RestResponse.data(ret);
    }

    /**
     * 统计
     * @return
     */
    @GetMapping("/stat")
    public RestResponse<ItemStatVO> stat() {
        ItemStatVO bo = itemService.stat();
        return RestResponse.data(bo);
    }
}

