package com.dayou.service.impl;

import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.StrUtil;
import com.dayou.common.BaseEntity;
import com.dayou.entity.Item;
import com.dayou.entity.ItemStage;
import com.dayou.enums.ItemStateEnum;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.ItemMapper;
import com.dayou.mapper.ItemStageMapper;
import com.dayou.service.IItemStageService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.vo.ItemStageVO;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.ibatis.annotations.Case;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dayou.utils.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

import org.springframework.transaction.annotation.Transactional;
import com.dayou.enums.BatchTaskTypeEnum;

import static com.dayou.exception.ErrorCode.STAGE_NODE_ERROR;
import static com.dayou.exception.ErrorCode.STAGE_NODE_ERROR1;

/**
 * <p>
 * 项目阶段表 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2022-12-08
 */
@Service
public class ItemStageServiceImpl extends ServiceImpl<ItemStageMapper, ItemStage> implements IItemStageService {


    @Autowired
    private ItemStageMapper itemStageMapper;

    @Autowired
    private ItemMapper itemMapper;

    @Override
    @SuppressWarnings("unchecked")
    public Page<ItemStage> selectPage(Page page,ItemStage itemStage){
        return this.page(page, new QueryWrapper<ItemStage>(itemStage));
    }


    @Override
    public ItemStage detail(Long id){
        return this.getById(id);
    }

    @Override
    public Boolean add(ItemStage itemStage){
        //判断阶段时间是否与已有阶段时间重复
        LocalDate startDate = itemStage.getStartDate();
        LocalDate endDate = itemStage.getEndDate();
        int check = itemStageMapper.checkDate(itemStage.getId(),itemStage.getItemId(),startDate,endDate);
        if (check!=0){
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"日期段与已存在的排期日期重复,请检查.");
        }
        if (StrUtil.isNotBlank(itemStage.getHeaderId())){
            itemStage.setFooterId(UUID.randomUUID().toString());
            this.update(new LambdaUpdateWrapper<ItemStage>()
                    .set(ItemStage::getHeaderId,itemStage.getFooterId())
                    .eq(ItemStage::getHeaderId,itemStage.getHeaderId()));
        }else {
            int count = this.count(new LambdaQueryWrapper<ItemStage>().eq(ItemStage::getItemId, itemStage.getItemId()));
            if (count>0){
                ErrorCode.throwBusinessException(STAGE_NODE_ERROR);
            }
            itemStage.setFooterId(UUID.randomUUID().toString());
        }
        return  this.save(itemStage);
    }

    @Transactional
    @Override
    public Boolean update(ItemStage itemStage){
        //判断阶段时间是否与已有阶段时间重复
        LocalDate startDate = itemStage.getStartDate();
        LocalDate endDate = itemStage.getEndDate();
        int check = itemStageMapper.checkDate(itemStage.getId(),itemStage.getItemId(),startDate,endDate);
        if (check!=0){
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"日期段与已存在的排期日期重复,请检查.");
        }
        if (StrUtil.isBlank(itemStage.getHeaderId())){
            ErrorCode.throwBusinessException(STAGE_NODE_ERROR);
        }
        if (StrUtil.isNotBlank(itemStage.getHeaderId()) &&
                StrUtil.isNotBlank(itemStage.getFooterId()) &&
                StrUtil.equals(itemStage.getHeaderId(),itemStage.getFooterId())){
            ErrorCode.throwBusinessException(STAGE_NODE_ERROR1);
        }
        //原节点headerId
        ItemStage oldStage = this.getById(itemStage.getId());

        ItemStage stage1 = this.getOne(new LambdaQueryWrapper<ItemStage>().eq(ItemStage::getHeaderId, itemStage.getHeaderId()));
        ItemStage stage2 = this.getOne(new LambdaQueryWrapper<ItemStage>().eq(ItemStage::getHeaderId, oldStage.getFooterId()));
        if (stage1!=null && stage1.getId() != itemStage.getId()){
            stage1.setHeaderId(oldStage.getFooterId());
            this.update(new LambdaUpdateWrapper<ItemStage>().set(ItemStage::getHeaderId,oldStage.getFooterId()).eq(ItemStage::getId,stage1.getId()));
        }
        if (stage2!=null){
            this.update(new LambdaUpdateWrapper<ItemStage>().set(ItemStage::getHeaderId,oldStage.getHeaderId()).eq(ItemStage::getId,stage2.getId()));
        }

        return  this.updateById(itemStage);
    }

    @Override
    @Transactional
    public Boolean delete(Long id){
        ItemStage itemStage = this.getById(id);
        if (itemStage.getHeaderId()==null){
            ItemStage secondStage = this.getOne(new LambdaQueryWrapper<ItemStage>().eq(ItemStage::getHeaderId, itemStage.getFooterId()));
            if (secondStage!=null){
                this.update(new LambdaUpdateWrapper<ItemStage>().set(ItemStage::getHeaderId,null).eq(ItemStage::getId,secondStage.getId()));
            }
        }else {
            ItemStage footerStage = this.getOne(new LambdaQueryWrapper<ItemStage>().eq(ItemStage::getHeaderId, itemStage.getFooterId()));
            ItemStage headerStage = this.getOne(new LambdaQueryWrapper<ItemStage>().eq(ItemStage::getFooterId, itemStage.getHeaderId()));
            if (footerStage!=null && headerStage!=null){
                this.update(new LambdaUpdateWrapper<ItemStage>().set(ItemStage::getHeaderId,headerStage.getFooterId()).eq(ItemStage::getId,footerStage.getId()));
            }
        }
        return this.removeById(id);
    }

    @Override
    public List<ItemStageVO> listByItemId(Long itemId) {
        List<ItemStageVO> stages = itemStageMapper.listByItemId(itemId);
        return toSort(stages,new LinkedList<>(),null);
    }

    @Override
    public List<List<ItemStageVO>> stageList(String state) {
        List<List<ItemStageVO>> list = new ArrayList<>();
        List<Long> itemIds = itemMapper.selectList(new LambdaQueryWrapper<Item>()
                        .eq(BaseEntity::getDeleted, false))
                .stream().map(BaseEntity::getId)
                .collect(Collectors.toList());
        itemIds.stream().forEach(x->{
            List<ItemStageVO> itemStageVOS = listByItemId(x);
            if (CollectionUtils.isNotEmpty(itemStageVOS)){
                LocalDate startDate = itemStageVOS.get(0).getStartDate();
                LocalDate endDate = itemStageVOS.get(itemStageVOS.size() - 1).getEndDate();
                LocalDate now = LocalDate.now();
                ItemStateEnum stateEnum = ItemStateEnum.getName(state);
                switch (stateEnum){
                    case PENDING:
                        if ((now.isAfter(startDate) || now.isEqual(startDate))
                                &&
                                (now.isBefore(endDate) || now.isEqual(endDate))){
                            list.add(itemStageVOS);
                        }
                        break;
                    case COMPLETED:
                        if (now.isAfter(endDate)){
                            list.add(itemStageVOS);
                        }
                        break;
                    case UNPLAYED:
                        if (now.isBefore(startDate)){
                            list.add(itemStageVOS);
                        }
                        break;
                    default:

                }
            }
        });
        return list;
    }

    /**
     * 递归排序
     */
    private List<ItemStageVO> toSort(List<ItemStageVO> stages,List<ItemStageVO> n ,ItemStageVO stageVO){
        if (stageVO == null){
            for (ItemStageVO stage : stages){
                if (StrUtil.isBlank(stage.getHeaderId())){
                    n.add(stage);
                    toSort(stages,n,stage);
                }
            }
        }
        for (ItemStageVO stage : stages){
            if (stageVO==null){
                return n;
            }
            if (StrUtil.equals(stage.getHeaderId(),stageVO.getFooterId())){
                n.add(stage);
                toSort(stages,n,stage);
            }
        }
        return null;
    }
}
